#include "glInit.h"

// Internal Variables
void (*userDisplayFunction)(void)=NULL;
void (*userReshapeFunction)(GLsizei width, GLsizei height)=NULL;
void (*userKeyboardFunction)(unsigned char key, int x, int y)=NULL;
void (*userTimer)(void)=NULL;
int userTimerInterval=0;

// Internal functions headers
void glInitDisplayFunction(void) ;
void glInitReshapeFunction(GLsizei width, GLsizei height);
void glInitKeyboardFunction(unsigned char key, int x, int y);
void glInitTimer(int value);

// ************************************************************************
// Function

int glWidth;
int glHeight;


// Create the main window
void CreateMainWindow(char *name, int width, int height, int top, int left)
{
    glutInitDisplayMode (GLUT_DOUBLE | GLUT_RGBA);
    glutInitWindowSize (width, height);
    glutInitWindowPosition (top, left);
    glutCreateWindow (name);

//	glEnable(GL_TEXTURE_2D);						// Enable Texture Mapping ( NEW )
//  glShadeModel(GL_SMOOTH);
//    glEnable(GL_DEPTH_TEST);						// Enables Depth
//    glDepthFunc(GL_LEQUAL);

    glEnable(GL_DEPTH_TEST);
    glDepthFunc(GL_LEQUAL);

    glutDisplayFunc (glInitDisplayFunction);
    glutReshapeFunc(glInitReshapeFunction);

    glWidth=width;
    glHeight=height;
}

// Set the function pointer to be called at display redraw
void SetUserDisplayFunction(void (*display)(void))
{
    // update pointer
    userDisplayFunction=display;
}

// Set the function pointer to be called at display redraw
void SetUserReshapeFunction(void (*reshape)(GLsizei width, GLsizei height))
{
    // update pointer
    userReshapeFunction=reshape;
}

void SetUserTimer(int milisecs, void (*timer)(void))
{
    userTimer=timer;
    userTimerInterval=milisecs;
    glutTimerFunc(userTimerInterval, glInitTimer, 1);
}


// ***********************************************************************
// Internal Functions
// Used to transparency management of screens

// Generic Display Function. Used to call user display function
void glInitDisplayFunction(void)
{
    // Basic startup
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    // Call user display function, if any
    if (userDisplayFunction!=NULL)
        userDisplayFunction();

    // Show drawn screen
    glutSwapBuffers ();
}

// Generic Reshape Function. Used to call user reshape function
void glInitReshapeFunction(GLsizei width, GLsizei height)
{
    glWidth=width;
    glHeight=height;

    // Call user reshape function, if any
    if (userReshapeFunction!=NULL)
        userReshapeFunction(width,height);
    else
        glutPostRedisplay();
}

// Generic Timer function
void glInitTimer(int value)
{
    if (userTimer!=NULL)
        userTimer();
    if (userTimerInterval>0)
        glutTimerFunc(userTimerInterval, glInitTimer, 1);
}
