#include "glTexture.h"

glTexture::glTexture(void)
{
    glGenTextures(1, &id);
    if (id==0)
        printf("[glTexture::glTexture] Error: Could not assign a Texture ID\n");
    transparent=false;
    width=height=1.0;
}

glTexture::~glTexture(void)
{
    glDeleteTextures(1, &id);
}

void glTexture::setTexture(Image *img)
{
    glBindTexture(GL_TEXTURE_2D, id);
    glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
    glTexImage2D(GL_TEXTURE_2D, 0, 4, img->getTextureWidth(), img->getTextureHeight(), 0, GL_RGBA, GL_UNSIGNED_BYTE, img->getPointer());
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);

    width=(float)img->getWidth()/(float)img->getTextureWidth();
    height=(float)img->getHeight()/(float)img->getTextureHeight();
}

void glTexture::setTransparent(bool value)
{
    transparent=value;
}

GLuint glTexture::getId(void)
{
    return id;
}

void glTexture::bindTexture(void)
{
    glEnable(GL_TEXTURE_2D);
    glShadeModel(GL_SMOOTH);
    glBindTexture(GL_TEXTURE_2D, id);

    if (transparent)
    {
        glEnable(GL_BLEND);
        glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
    }
    else
        glDisable(GL_BLEND);
}

void glTexture::unbindTexture(void)
{
    glDisable(GL_TEXTURE_2D);
    glDisable(GL_BLEND);
}

float glTexture::getWidth(void)
{
    return width;
}

float glTexture::getHeight(void)
{
    return height;
}
