#include "board.h"

Board::Board(void)
{
    int i,j;
    for (i=0;i<10;i++)
        for (j=0;j<8;j++)
            pieces[i][j]=(PieceOnBoard){true,0,0,0};
    turnPlayer=-1;
    playerId=-1;
    gameId=-1;
    playerNumber=-1;
    laser=NULL;
}

Board::~Board(void)
{
    if (laser)
        delete laser;
}

void Board::setGameId(int gameId)
{
    this->gameId=gameId;
}

void Board::setPlayerId(int playerId)
{
    this->playerId=playerId;
}

int Board::getGameId(void)
{
    return gameId;
}

int Board::getPlayerId(void)
{
    return playerId;
}

void Board::setPlayerNumber(int playerNumber)
{
    this->playerNumber=playerNumber;
}

int Board::getPlayerNumber(void)
{
    return playerNumber;
}

int Board::getTurnPlayer(void)
{
    return turnPlayer;
}

void Board::askServer(bool update)
{
    char buffer[65536];


    if (playerNumber==turnPlayer)
        return;

    Net *net=new Net();
    net->setServer(ServerName,ServerPort);
    sprintf(buffer,"<REQUEST><RT>GAME BOARD</RT><GID>%i</GID><PLAYER>%i</PLAYER></REQUEST>",gameId,playerId);
    if (net->sendRequest(buffer))
    {
        XMLParser *serverAnswer=new XMLParser(net->getAnswer(),"<SERVER ANSWER>");

        XMLParser *specificAnswer=new XMLParser(serverAnswer->getContent(),"<SPECIFIC ANSWER>");

        if (laser)
            delete laser;
        laser=new XMLParser(specificAnswer->getContent(),"<GAME INFO>");

        XMLParser *gameCurrentBoard=new XMLParser(specificAnswer->getContent(),"<GAME CURRENT BOARD>");

        XMLParser *gameStatus=new XMLParser(serverAnswer->getContent(),"<GAME STATUS>");
        XMLParser *turn=new XMLParser(gameStatus->getContent(),"<PLAYERNUMBER>");
        turnPlayer=atoi(turn->getContent())-1;

        delete gameStatus;
        delete turn;

        delete serverAnswer;
        delete specificAnswer;

        char *tmp=gameCurrentBoard->getContent();
        int i=0;

        while (*tmp!='\0')
        {
            tmp++; // Pula <

            if (*tmp==' ')
            {
                tmp+=2; // " >"
                piecesNew[i%10][i/10]=(PieceOnBoard){true,0,0,0};
            }
            else
            {
                piecesNew[i%10][i/10]=translatePiece(tmp);
                tmp += 6; //"OPhaO>"
            }
            i++;
        }
        delete gameCurrentBoard;
        if (update)
            updateBoard();
    }
    else
        printf("[Board::askServer] Error contacting server\n");
}

void Board::updateBoard(void)
{
    int i,j;
    for (i=0;i<10;i++)
        for (j=0;j<8;j++)
            pieces[i][j]=piecesNew[i][j];
}


PieceOnBoard Board::getPiece(int i, int j,bool old)
{
    if (i>=0 && i<=9 && j>=0 && j<=7)
    {
        if (old)
            return pieces[i][j];
        else
            return piecesNew[i][j];
    }
    else
        return (PieceOnBoard){true,0,0,0};
}

bool Board::doMove(int i1,int j1, int i2, int j2)
{
    bool ret=false;
    char buffer[1024];

    Net *net=new Net();
    net->setServer(ServerName,ServerPort);
    sprintf(buffer,"<REQUEST><RT>MOVE PIECE</RT><GID>%i</GID><PLAYER>%i</PLAYER><X>%i</X><Y>%i</Y><X2>%i</X2><Y2>%i</Y2></REQUEST>",gameId,playerId,j1,i1,j2,i2);

    if (net->sendRequest(buffer))
    {
        XMLParser *serverAnswer=new XMLParser(net->getAnswer(),"<SERVER ANSWER>");
        XMLParser *status=new XMLParser(serverAnswer->getContent(),"<STATUS>");
        if (strcmp(status->getContent(),"OK")==0)
        {
            ret=true;
            turnPlayer=-1;
            askServer(false);
        }
        delete serverAnswer;
        delete status;
    }
    else
        printf("[Board::doMove] Error contacting server\n");

    delete net;
    return ret;
}

bool Board::doRotate(int i1,int j1, int sense)
{
    bool ret=false;
    char buffer[1024];

    Net *net=new Net();
    net->setServer(ServerName,ServerPort);
    sprintf(buffer,"<REQUEST><RT>ROTATE PIECE</RT><GID>%i</GID><PLAYER>%i</PLAYER><X>%i</X><Y>%i</Y><SENSE>%s</SENSE></REQUEST>",
        gameId,playerId,j1,i1,(sense==RotateClockWise ? "CLOCKWISE" : "ANTICLOCKWISE"));
    if (net->sendRequest(buffer))
    {
        XMLParser *serverAnswer=new XMLParser(net->getAnswer(),"<SERVER ANSWER>");
        XMLParser *status=new XMLParser(serverAnswer->getContent(),"<STATUS>");
        if (strcmp(status->getContent(),"OK")==0)
        {
            ret=true;
            turnPlayer=-1;
            askServer(false);
        }
        delete serverAnswer;
        delete status;
    }
    else
        printf("[Board::doRotate] Error contacting server\n");

    delete net;
    return ret;
}


char *Board::getLaser(void)
{
    if (laser)
        return laser->getContent();
    return "<LASER></LASER>";
}

int Board::piecesOnBoard(int Player,bool old)
{
    PieceOnBoard p;
    int count=0;
    int i,j;
    for (i=0;i<10;i++)
        for (j=0;j<10;j++)
        {
            if (old)
                p=pieces[i][j];
            else
                p=piecesNew[i][j];

            if (!p.empty && Player==PlayerBoth)
                count++;
            else if (!p.empty && p.player==Player)
                count++;
        }
    return count;
}

PieceOnBoard Board::translatePiece(char *pieceText)
{
    PieceOnBoard p;
    p.empty=false;
    if (*pieceText++=='X')
        p.player=PlayerGolden;
    else
        p.player=PlayerSilver;

    if (*pieceText=='O')
    {
        p.piece=PieceObelisk;
        p.orientation=OrientationNone;
    }
    else if (*pieceText=='P' && *(pieceText+1)=='h')
    {
        p.piece=PiecePharaoh;
        p.orientation=OrientationNone;
    }
    else
    {
        if (*pieceText++=='D')
            p.piece=PieceDjed;
        else
            p.piece=PiecePyramid;

        if (*pieceText=='$')
        {
            if (*(pieceText+1)=='/')
                p.orientation=OrientationSE;
            else
                p.orientation=OrientationNE;
        }
        else
        {
            if (*pieceText=='/')
                p.orientation=OrientationNW;
            else
                p.orientation=OrientationSW;
        }
    }
    return p;
}

PieceOnBoard Board::getDestroyedPiece(int *i, int *j)
{
    char dir;
    char *tmp=getLaser();
    PieceOnBoard p=(PieceOnBoard){true,0,0,0};
    if (strlen(tmp)>19)
    {
        tmp += strlen(tmp)-19; //"0 0 D;XPhaX</LASER>
        if (*tmp!=',') // There is a destroyed Piece!!!
        {
            sscanf(tmp,"%i,%i %c",j,i,&dir);
            tmp+=6;
            p=translatePiece(tmp);
        }
    }

    return p;
}
