#include "gameScreen.h"

extern glSprite *sprRotate;

GlIGameScreen::GlIGameScreen(void)
{
    mouseCursor->chooseCursor(0);
    glutSetCursor(GLUT_CURSOR_NONE);
    game=-1;
    playerId=-1;
    selected=-1;
    animate=false;
    board=new Board();
    boardDraw=new BoardDraw(0,64);
}

GlIGameScreen::~GlIGameScreen(void)
{
    delete board;
    delete boardDraw;
}

// ***********************************************************************
// Internal Functions
// Used to transparency management of screens

// Basic reshape function. 1 pixel per point
void GlIGameScreen::reshape(int width, int height)
{
    GLInterface::reshape(width,height);
}

// Display start screen message
void GlIGameScreen::display(void)
{
    char buffer[1024];
    void *font=GLUT_BITMAP_TIMES_ROMAN_24;
    static int animateStep=-1;

    int i,j,id,jd;
    PieceOnBoard p,pn,pd;
    glLoadIdentity();

    boardDraw->draw();

    mouseCursor->chooseCursor(0);

    if (animate)
    {
        id=jd=-1;
        if (animateStep==-1)
            animateStep=0;
        else
        {
            animateStep++;
//            if (animateStep>=GameFPS*2)
            if (animateStep>=GameFPS)
            {
                animate=false;
                board->updateBoard();
            }
            pd=board->getDestroyedPiece(&id,&jd);
        }


    }
    else if (animateStep>=0)
        animateStep=-1;

    for (i=0;i<10;i++)
        for (j=0;j<8;j++)
        {
            p=board->getPiece(i,j);
            pn=board->getPiece(i,j,false);
            if (i==id && j==jd)
                pn=pd;

            if (!animate && !p.empty)
                PieceDraw::draw(boardDraw->getXOf(i,j),boardDraw->getYOf(i,j),
                    p.player,p.piece,p.orientation);
//            else if (animate && animateStep>=GameFPS && !pn.empty)
            else if (animate && animateStep>=15 && !pn.empty)
                PieceDraw::draw(boardDraw->getXOf(i,j),boardDraw->getYOf(i,j),
                    pn.player,pn.piece,pn.orientation);
//            else if (animate && animateStep<GameFPS)
            else if (animate && animateStep<15)
            {
                if (p.player==pn.player && p.piece==pn.piece && p.orientation==pn.orientation && !p.empty && !pn.empty)
                    PieceDraw::draw(boardDraw->getXOf(i,j),boardDraw->getYOf(i,j),
                        p.player,p.piece,p.orientation);
                else
                {
                    if (!p.empty)
                    {
                        PieceDraw::draw(boardDraw->getXOf(i,j),boardDraw->getYOf(i,j),
//                            p.player,p.piece,p.orientation,(GameFPS-animateStep)*255/GameFPS);
                            p.player,p.piece,p.orientation,(15-animateStep)*255/15);
                    }
                    if (!pn.empty)
                    {
                        PieceDraw::draw(boardDraw->getXOf(i,j),boardDraw->getYOf(i,j),
//                            pn.player,pn.piece,pn.orientation,(animateStep)*255/GameFPS);
                            pn.player,pn.piece,pn.orientation,(animateStep)*255/15);
                    }
                }
            }
        }

    if (selected>=0)
    {
        static int glow=0;
        p=board->getPiece(selected%10,selected/10);
        PieceDraw::draw(boardDraw->getXOf(selected%10,selected/10),
                        boardDraw->getYOf(selected%10,selected/10),
                        p.player,p.piece,p.orientation,(glow<127 ? 255-glow : glow));
        if (p.piece==PieceDjed || p.piece==PiecePyramid)
        {
            int mx,my,x;
            mousePosition(&mx,&my);
            i=boardDraw->getIOf(mx,my);
            j=boardDraw->getJOf(mx,my);
            x=mx-boardDraw->getXOf(i,j);
            glColor3f(.7,.7,.7);
            if (i==selected%10 && j==selected/10 && x<16)
                glColor3f(1,1,1);
            sprRotate->draw(boardDraw->getXOf(selected%10,selected/10),
                            boardDraw->getYOf(selected%10,selected/10),0,0);
            glColor3f(.7,.7,.7);
            if (i==selected%10 && j==selected/10 && x>47)
                glColor3f(1,1,1);
            sprRotate->draw(boardDraw->getXOf(selected%10,selected/10)+32,
                            boardDraw->getYOf(selected%10,selected/10),1,0);
        }
        glow = (glow+3)%256;
    }

    if (animate && animateStep>=15)
    {
//        drawLaser(board->getLaser(),animateStep);
        drawLaser(board->getLaser(),animateStep-15);
    }

    glColor3f(1,1,1);
    sprintf(buffer,"Deflexion Game #%i",board->getGameId());
    glTextString((glWidth-glTextStringLength(font,buffer))/2,32,0,font,buffer);

    font=GLUT_BITMAP_HELVETICA_18;

    if (board->getPlayerNumber()==PlayerGolden)
    {
        glColor3f(1,1,0);
        sprintf(buffer,"Golden Player");
        glTextString((glWidth-glTextStringLength(font,buffer))/2,54,0,font,buffer);
    }
    else if (board->getPlayerNumber()==PlayerSilver)
    {
        glColor3f(.7,.7,.8);
        sprintf(buffer,"Silver Player");
        glTextString((glWidth-glTextStringLength(font,buffer))/2,54,0,font,buffer);
    }

    glColor3f(1,1,1);
    if (board->getPlayerNumber()==board->getTurnPlayer())
        glTextString(boardDraw->getXOf(0,8)+32,boardDraw->getYOf(0,8)+44,0,font,"Your turn");
    else
        glTextString(boardDraw->getXOf(0,8)+32,boardDraw->getYOf(0,8)+44,0,font,"Adversary turn");

    chooseMousePointer();

    // Draw mouse
    if (mouseCursor)
        mouseCursor->draw();
}

void GlIGameScreen::keyboard(int key, bool release)
{
    // Entry point for game start
    // Call initialize function, if any
}

void GlIGameScreen::mouse(int action)
{
    int mx,my;
    int i,j;
    int x;
    PieceOnBoard p;
    if (action==MOUSE_BUTTON_PRESSED)
    {
        mousePosition(&mx,&my);
        i=boardDraw->getIOf(mx,my);
        j=boardDraw->getJOf(mx,my);

        if (i>=0 && i<=9 && j>=0 && j<=7)
        {
            if (selected>=0)
            {
                if (i==selected%10 && j==selected/10)
                {
                    x=mx-boardDraw->getXOf(i,j);
                    if (x<16)
                    {
                        if (board->doRotate(selected%10,selected/10,RotateAntiClockWise))
                        {
                            animate=true;
                            selected=-1;
                        }
                    }
                    else if (x>=48)
                    {
                        if (board->doRotate(selected%10,selected/10,RotateClockWise))
                        {
                            animate=true;
                            selected=-1;
                        }
                    }
                    else
                        selected=-1;
                }
                else if (abs(i-selected%10)<=1 && abs(j-selected/10)<=1)
                {
                    if (board->doMove(selected%10,selected/10,i,j))
                    {
                        animate=true;
                        selected=-1;
                    }
                }
            }
            else
            {
                p=board->getPiece(i,j);
                if (!p.empty && board->getPlayerNumber()==board->getTurnPlayer() && board->getPlayerNumber()==p.player)
                        selected=j*10+i;
            }
        }
    }
    glutPostRedisplay();
}

void GlIGameScreen::timer(void)
{
    static int counter;

    counter++;
    if (counter%(GameFPS*5)==0 && !animate)
    {
        int turn=board->getTurnPlayer();
        board->askServer(false);
        if (turn!=board->getTurnPlayer())
            animate=true;
        else
            board->updateBoard();
    }

    if (selected>=0 || animate)
        glutPostRedisplay();
}

void GlIGameScreen::setGame(int game)
{
    this->game=game;
    board->setGameId(game);
}

void GlIGameScreen::setPlayerId(int playerId)
{
    this->playerId=playerId;
    board->setPlayerId(playerId);
}

void GlIGameScreen::setPlayerNumber(int playerNumber)
{
    this->playerNumber=playerNumber;
    board->setPlayerNumber(playerNumber);
}


void GlIGameScreen::chooseMousePointer(void)
{
    PieceOnBoard p;
    int i,j;
    int x;
    int mx,my;

    mousePosition(&mx,&my);

    i=boardDraw->getIOf(mx,my);
    j=boardDraw->getJOf(mx,my);

    if (selected<0)
    {
        if (i>=0 && i<=9 && j>=0 && j<=7)
        {
            p=board->getPiece(i,j);
            if (!p.empty)
            {
                if (board->getPlayerNumber()==board->getTurnPlayer() && board->getPlayerNumber()==p.player)
                    mouseCursor->chooseCursor(1);
                else
                    mouseCursor->chooseCursor(3);
            }
        }
    }
    else
    {
        if (i==selected%10 && j==selected/10)
        {
            x=mx-boardDraw->getXOf(i,j);
            if (x<16 || x>=48)
                mouseCursor->chooseCursor(2);
            else
                mouseCursor->chooseCursor(1);
        }
        else if (abs(i-selected%10)<=1 && abs(j-selected/10)<=1)
            mouseCursor->chooseCursor(2);
        else
            mouseCursor->chooseCursor(3);
    }
}

void GlIGameScreen::drawLaser(char *laser, int step)
{
    int j,i;
    char dir;
    int x,y,x1,y1,tmp;
    float radius;
    PieceOnBoard p;

    laser += 7; // Jump "<LASER>

    if (*laser==' ' || *laser=='<')
    {
        return;
    }

    sscanf(laser,"%i,%i %c",&j,&i,&dir);
    x1=x=boardDraw->getXOf(i,j)+31;
    y1=y=boardDraw->getYOf(i,j)+(j>0?62:1);

    p=board->getDestroyedPiece(&i,&j);
    if (!p.empty)
    {
//        tmp=GameFPS*2-step;
        tmp=45-step;
        tmp=(tmp>20 ? 20 : tmp);
        PieceDraw::draw(boardDraw->getXOf(i,j),boardDraw->getYOf(i,j),
            p.player,p.piece,p.orientation,tmp*255/20);
    }

    glColor3f(1,0,0);

    glBegin(GL_LINES);
    tmp=step;

    while(step -- && *laser!='<' && *laser)
    {
        sscanf(laser,"%i,%i %c",&j,&i,&dir);
        x1=boardDraw->getXOf(i,j)+31;
        y1=boardDraw->getYOf(i,j)+31;
        glVertex3f(x,y,0);
        glVertex3f(x1,y1,0);
        glVertex3f(x+1,y+1,0);
        glVertex3f(x1+1,y1+1,0);
        x=x1;
        y=y1;
        laser+=6;
    }

    if (step>0)
    {
        // Check if no Piece died
        if (p.empty)
        {
            x1+=(dir=='W' ? -32 : (dir=='E' ? 32 : 0));
            y1+=(dir=='N' ? -32 : (dir=='S' ? 32 : 0));
            glVertex3f(x,y,0);
            glVertex3f(x1,y1,0);
            glVertex3f(x+1,y+1,0);
            glVertex3f(x1+1,y1+1,0);
        }
        else
        {
            if (step>20)
                step=20;
//            if (tmp>GameFPS*2-20)
//                step=GameFPS*2-tmp;
            if (tmp>45-20)
                step=45-tmp;
            radius=step/10.0;

            glEnd();


            glColor3f(1,0,0);

            glBegin(GL_POLYGON);
                glVertex3f(x-10.0*radius,y            ,0);
                glVertex3f(x-7.07*radius,y-7.07*radius,0);
                glVertex3f(x            ,y-10.0*radius,0);
                glVertex3f(x+7.07*radius,y-7.07*radius,0);
                glVertex3f(x+10.0*radius,y            ,0);
                glVertex3f(x+7.07*radius,y+7.07*radius,0);
                glVertex3f(x            ,y+10.0*radius,0);
                glVertex3f(x-7.07*radius,y+7.07*radius,0);
        }
    }
    glEnd();
}
