#include "startScreen.h"

GlIStartScreen::GlIStartScreen(void)
{
    gameListSize=-1;

    gs=NULL;

    mouseOption=-3;
    mouseCursor->chooseCursor(0);
    glutSetCursor(GLUT_CURSOR_NONE);

    firstDraw=false;
}

GlIStartScreen::~GlIStartScreen(void)
{
//    delete texBackground;
}

void GlIStartScreen::getGameList(void)
{
    gameListSize=0;
    Net *net=new Net();
    net->setServer(ServerName,ServerPort);
    if (net->sendRequest("<REQUEST><RT>LIST GAMES</RT><GT>deflexion</GT><GS>all</GS></REQUEST>"))
    {
        XMLParser *serverAnswer=new XMLParser(net->getAnswer(),"<SERVER ANSWER>");
        XMLParser *specificAnswer=new XMLParser(serverAnswer->getContent(),"<SPECIFIC ANSWER>");
        XMLParser *listGames=new XMLParser(specificAnswer->getContent(),"<LIST GAMES>");
        delete specificAnswer;
        delete serverAnswer;

        XMLParser *game;

        game=new XMLParser(listGames->getContent(),"<GAME>",gameListSize);

        while (strlen(game->getContent())>0)
        {
            XMLParser *tmp=new XMLParser(game->getContent(),"<GID>");
            sscanf(tmp->getContent(),"%i",&gameList[gameListSize].gameId);
            delete tmp;

            tmp=new XMLParser(game->getContent(),"<#PLAYERS>");
            sscanf(tmp->getContent(),"%i",&gameList[gameListSize].numPlayers);
            delete tmp;

            tmp=new XMLParser(game->getContent(),"<WP>");
            gameList[gameListSize].needPlayers=
                (strcmp(tmp->getContent(),"true")==0);
            delete tmp;

            tmp=new XMLParser(game->getContent(),"<ENDED>");
            gameList[gameListSize].ended=
                (strcmp(tmp->getContent(),"true")==0);
            delete tmp;

            gameListSize++;
            delete game;
            game=new XMLParser(listGames->getContent(),"<GAME>",gameListSize);
        }

        delete listGames;
    }
    else
        printf("[GlIStarScreen::getGameList] Could not get game list\n");

    delete net;

}

// ***********************************************************************
// Internal Functions
// Used to transparency management of screens

// Basic reshape function. 1 pixel per point
void GlIStartScreen::reshape(int width, int height)
{
    GLInterface::reshape(width,height);
}

// Display start screen message
void GlIStartScreen::display(void)
{
    int mx,my;
    void *font=GLUT_BITMAP_TIMES_ROMAN_24;
    char *message;

    firstDraw=true;

    glLoadIdentity();

    mouseCursor->chooseCursor(0);
    mouseOption=-3;

    mousePosition(&mx,&my);

    glColor3f(1,1,1);
    message="DEFLEXION Game";
    glTextString((glWidth-glTextStringLength(font,message))/2,50,0,font,message);

    font=GLUT_BITMAP_HELVETICA_18;

    if (gameListSize==-1)
    {
        message="Getting server game list";
        glTextString((glWidth-glTextStringLength(font,message))/2,100,0,font,message);
    }
    else
    {
        char buffer[256];
        int y=100,c;
        message="Refresh Game List";
        glTextString(100,y,0,font,message);

        if (mx>100 && mx<100+glTextStringLength(font,message) && my<=y && my>=y-22)
        {
            mouseOption=-1;
            mouseCursor->chooseCursor(1);
        }


        y+=44;
        message="Create new game";
        glTextString(100,y,0,font,message);

        if (mx>100 && mx<100+glTextStringLength(font,message) && my<=y && my>=y-22)
        {
            mouseOption=-2;
            mouseCursor->chooseCursor(1);
        }


        y+=44;
        if (gameListSize==0)
        {
                glTextString(100,y,0,font,"No games in server");
        }
        else
            for (c=0;c<gameListSize;c++)
            {
                if (gameList[c].ended)
                    glColor3f(1,0,0);
                else if (gameList[c].needPlayers)
                    glColor3f(0,1,0);
                else
                    glColor3f(1,1,0);

                sprintf(buffer,"Game #%i (%i Players)",gameList[c].gameId,gameList[c].numPlayers);
                glTextString(100,y,0,font,buffer);

                glColor3f(1,1,1);
                if (gameList[c].needPlayers)
                {
                    glTextString(300,y,0,font,"Join");

                    if (mx>300 && mx<300+glTextStringLength(font,message) && my<=y && my>=y-22)
                    {
                        mouseOption=c;
                        mouseCursor->chooseCursor(1);
                    }
                }

                y+=22;
            }
    }

    // Draw mouse
    if (mouseCursor)
        mouseCursor->draw();
}

void GlIStartScreen::keyboard(int key, bool release)
{
    // Entry point for game start
    // Call initialize function, if any
}

void GlIStartScreen::mouse(int action)
{
    char buffer[1024];
    if (action==MOUSE_BUTTON_PRESSED)
    {
        if (mouseOption==-1)
            gameListSize=-1;
        else if (mouseOption==-2)
        {
            Net *net=new Net();
            net->setServer(ServerName,ServerPort);
            sprintf(buffer,"<REQUEST><RT>NEW GAME</RT><GT>deflexion</GT></REQUEST>");

            if (net->sendRequest(buffer))
            {
                XMLParser *serverAnswer=new XMLParser(net->getAnswer(),"<SERVER ANSWER>");
                XMLParser *status=new XMLParser(serverAnswer->getContent(),"<STATUS>");
                if (strcmp(status->getContent(),"OK")==0)
                {
                    gameListSize=-1;
                }
                else
                    printf("Server didn't accept I create a game\n");
            }
            delete net;
        }
        else if (mouseOption>=0 && mouseOption<gameListSize)
        {
            Net *net=new Net();
            net->setServer(ServerName,ServerPort);

            int player=random()%1000;

            sprintf(buffer,"<REQUEST><RT>JOIN GAME</RT><GID>%i</GID><PLAYER>%i</PLAYER></REQUEST>",gameList[mouseOption].gameId,player);
            if (net->sendRequest(buffer))
            {
                XMLParser *serverAnswer=new XMLParser(net->getAnswer(),"<SERVER ANSWER>");
                XMLParser *status=new XMLParser(serverAnswer->getContent(),"<STATUS>");
                XMLParser *specificAnswer=new XMLParser(serverAnswer->getContent(),"<SPECIFIC ANSWER>");
                XMLParser *playerNumber=new XMLParser(specificAnswer->getContent(),"<PLAYERNUMBER>");
                if (strcmp(status->getContent(),"OK")==0)
                {
                    gs=new GlIGameScreen();
                    gs->setPreviousInterface(this);
                    gs->setGame(gameList[mouseOption].gameId);
                    gs->setPlayerId(player);
                    gs->setPlayerNumber(atoi(playerNumber->getContent())-1);
                    setInterface(gs);
                }
                else
                    printf("Server didn't accept I in game #%i\n",gameList[mouseOption].gameId);
                delete status;
                delete serverAnswer;
            }
            else
                printf("Could not join game #%i\n",gameList[mouseOption].gameId);
            delete net;

        }
    }
    glutPostRedisplay();
}

void GlIStartScreen::timer(void)
{
    if (gs)
    {
        delete gs;
        gs=NULL;
    }

    if (gameListSize==-1 && firstDraw)
    {
        mouseOption=-3;
        getGameList();
    }
}
